package main

import (
	"container/list"
)

type Edge interface {
	// Source() Node
	Target() Node
	Length() int
}

type Node interface {
	Edges() []Edge
	Index() interface{}
}

// breadth-first-search functionality
type BFS struct {
	queue     list.List
	distances map[interface{}]int
}

func NewBFS() *BFS {
	bfs := &BFS{}
	bfs.distances = make(map[interface{}]int)
	return bfs
}

// TODO: Consider creating a specialized BFS for 'int' types

// Add a location to the search queue, this will check for duplicates first
func (bfs *BFS) Add(loc Node, dist int) {
	if d, present := bfs.distances[loc.Index()]; present {
		if d > dist {
			bfs.distances[loc.Index()] = dist
		}
		return
	}
	bfs.queue.PushBack(loc)
	bfs.distances[loc.Index()] = dist
}

func (bfs *BFS) Next() (node Node, dist int) {
	first := bfs.queue.Front()
	if first == nil {
		return nil, 0
	}
	bfs.queue.Remove(first)
	node = first.Value.(Node)
	dist = bfs.distances[node.Index()]
	edges := node.Edges()
	for i := range edges {
		bfs.Add(edges[i].Target(), 0)
	}
	return node, dist
}
