package main

type DenseIntSet struct {
	bits []uint32
}

// Determine what byte and what bit a given value maps to
func offset(v uint) (o uint, m uint32) {
	o = v >> 5
	m = 1 << (v & 0x1F)
	return
}

// Ensure that we have a long enough 'bits' vector to handle a given offset
func (set *DenseIntSet) ensureBits(o uint) {
	for i := uint(len(set.bits)); i <= o; i++ {
		set.bits = append(set.bits, 0)
	}
}
// Add a value to the set
func (set *DenseIntSet) Add(v uint) {
	o, m := offset(v)
	set.ensureBits(o)
	set.bits[o] |= m
}

func (set *DenseIntSet) innerContains(v uint) (contained bool, o uint, m uint32) {
	o, m = offset(v)
	if o >= uint(len(set.bits)) {
		contained = false
		return
	}
	contained = ((set.bits[o] & m) == m)
	return
}
// Is this value in the set?
func (set *DenseIntSet) Contains(v uint) bool {
	contained, _, _ := set.innerContains(v)
	return contained
}

// Remove an entry from the set, return True if the entry existed
func (set *DenseIntSet) Remove(v uint) bool {
	contained, o, m := set.innerContains(v)
	if !contained {
		return false
	}
	val := set.bits[o]
	set.bits[o] = (val & ^m)
	return true
}
