package main

import (
	"testing"
)

func TestMap(t *testing.T) {
	m := NewMap(4, 3)
	if m.String() != `. . . 
. . . 
. . . 
. . . 
` {
		t.Errorf("map is wrong size, got `%s`", m)
	}

	loc := m.FromRowCol(3, 2)
	row, col := m.FromLocation(loc)
	if row != 3 || col != 2 {
		t.Errorf("conversion broken, got (%v, %v), wanted (3, 2)", row, col)
	}

	loc2 := m.FromRowCol(3, -1)
	if loc2 != loc {
		t.Errorf("from xy broken, got (%v), wanted (%v)", loc2, loc)
	}

	n := m.FromRowCol(2, 2)
	s := m.FromRowCol(4, 2)
	e := m.FromRowCol(3, 3)
	w := m.FromRowCol(3, 1)

	if n != m.Move(loc, North) {
		t.Errorf("Move north is broken")
	}
	if s != m.Move(loc, South) {
		t.Errorf("Move south is broken")
	}
	if e != m.Move(loc, East) {
		t.Errorf("Move east is broken")
	}
	if w != m.Move(loc, West) {
		t.Errorf("Move west is broken")
	}

	m.AddAnt(n, MY_ANT)
	m.AddAnt(s, ANT_1)
	m.AddAnt(w, MY_HILL)
	m.AddAnt(e, MY_OCCUPIED_HILL)
	m.AddHill(m.FromRowCol(0, 0), HILL_1)
	m.AddHill(m.FromRowCol(1, 0), HILL_1)
	m.AddAnt(m.FromRowCol(1, 0), ANT_1)

	if m.String() != `1 . b 
B . . 
. . a 
A 0 . 
` {
		t.Errorf("map put ants in wrong place, got `%s`", m)
	}
}

func TestMapNodeIsNode(t *testing.T) {
	_ = Node(&MapNode{})
}

func TestMapEdgeIsEdge(t *testing.T) {
	_ = Edge(&MapEdge{})
}

func TestMapNode(t *testing.T) {
	m := NewMap(4, 3)
	foodLoc := m.FromRowCol(2, 1)
	m.AddFood(foodLoc)
	if m.String() != `. . . 
. . . 
. * . 
. . . 
` {
		t.Fatalf("unexpected map, got: `%s`", m)
	}
	mn := &m.grid[foodLoc]
	edges := mn.Edges()
	if edges == nil || len(edges) != 4 ||
		edges[0].Target().(*MapNode).loc != m.FromRowCol(1, 1) ||
		edges[1].Target().(*MapNode).loc != m.FromRowCol(2, 0) ||
		edges[2].Target().(*MapNode).loc != m.FromRowCol(2, 2) ||
		edges[3].Target().(*MapNode).loc != m.FromRowCol(3, 1) {
		t.Fatalf("Incorrect edges, got %v", edges)
	}
}

func TestMapNodeEdgesNotWater(t *testing.T) {
	m := NewMap(4, 3)
	foodLoc := m.FromRowCol(2, 1)
	waterLoc := m.FromRowCol(3, 1)
	m.AddFood(foodLoc)
	m.AddWater(waterLoc)
	if m.String() != `. . . 
. . . 
. * . 
. % . 
` {
		t.Fatalf("unexpected map, got: `%s`", m)
	}
	mn := &m.grid[foodLoc]
	edges := mn.Edges()
	if edges == nil || len(edges) != 3 ||
		edges[0].Target().(*MapNode).loc != m.FromRowCol(1, 1) ||
		edges[1].Target().(*MapNode).loc != m.FromRowCol(2, 0) ||
		edges[2].Target().(*MapNode).loc != m.FromRowCol(2, 2) {
		t.Fatalf("Incorrect edges, got %v", edges)
	}
}

func TestMapNodeEdgesWrapping(t *testing.T) {
	m := NewMap(4, 3)
	tl := m.FromRowCol(0, 0)
	tr := m.FromRowCol(0, 2)
	bl := m.FromRowCol(3, 0)
	br := m.FromRowCol(3, 2)
	edges := m.grid[tl].Edges()
	if edges == nil || len(edges) != 4 ||
		edges[0].Target().(*MapNode).loc != bl ||
		edges[1].Target().(*MapNode).loc != tr ||
		edges[2].Target().(*MapNode).loc != m.FromRowCol(0, 1) ||
		edges[3].Target().(*MapNode).loc != m.FromRowCol(1, 0) {
		t.Fatalf("Incorrect edges for top left, got %v", edges)
	}
	edges = m.grid[tr].Edges()
	if edges == nil || len(edges) != 4 ||
		edges[0].Target().(*MapNode).loc != br ||
		edges[1].Target().(*MapNode).loc != m.FromRowCol(0, 1) ||
		edges[2].Target().(*MapNode).loc != tl ||
		edges[3].Target().(*MapNode).loc != m.FromRowCol(1, 2) {
		t.Fatalf("Incorrect edges for top right, got %v", edges)
	}
	edges = m.grid[bl].Edges()
	if edges == nil || len(edges) != 4 ||
		edges[0].Target().(*MapNode).loc != m.FromRowCol(2, 0) ||
		edges[1].Target().(*MapNode).loc != m.FromRowCol(3, 2) ||
		edges[2].Target().(*MapNode).loc != m.FromRowCol(3, 1) ||
		edges[3].Target().(*MapNode).loc != m.FromRowCol(0, 0) {
		t.Fatalf("Incorrect edges for bottom left, got %v", edges)
	}
	edges = m.grid[br].Edges()
	if edges == nil || len(edges) != 4 ||
		edges[0].Target().(*MapNode).loc != m.FromRowCol(2, 2) ||
		edges[1].Target().(*MapNode).loc != m.FromRowCol(3, 1) ||
		edges[2].Target().(*MapNode).loc != bl ||
		edges[3].Target().(*MapNode).loc != tr {
		t.Fatalf("Incorrect edges for bottom right, got %v", edges)
	}
}
